"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.declareUserRepositoryTests = void 0;
const chai_1 = require("chai");
// eslint-disable-next-line import/no-extraneous-dependencies
const uuid_1 = require("uuid");
/* eslint-disable @typescript-eslint/no-unused-expressions */
function declareNegativeTestsForGetUserByIdMethod(getUserByID, existingUsers) {
    it(`[${getUserByID.name}] Wrong format id`, async () => {
        getUserByID('some random string')
            .then(() => {
            throw new Error('was not supposed to succeed');
        })
            .catch((err) => {
            (0, chai_1.expect)(err).to.exist;
        });
    });
    it(`[${getUserByID.name}] Non existing user`, async () => {
        let user;
        if (typeof existingUsers[0].id === 'number') {
            user = await getUserByID(99999);
        }
        else {
            user = await getUserByID((0, uuid_1.v4)());
        }
        (0, chai_1.expect)(user).not.to.exist;
    });
}
function declareInvitationTests(getUserByID, existingUsers) {
    declareNegativeTestsForGetUserByIdMethod(getUserByID, existingUsers);
    it(`[${getUserByID.name}] Get non confirmed user by ID`, async function test() {
        const nonConfirmedUser = existingUsers.find((user) => !user.hash);
        if (!nonConfirmedUser) {
            return this.skip();
        }
        const user = await getUserByID(nonConfirmedUser.id);
        (0, chai_1.expect)(user).to.exist;
        return (0, chai_1.expect)(user?.id).to.eq(nonConfirmedUser.id);
    });
}
function declareUserRepositoryTests(userRepo, existingUsers) {
    // test if there are some users seeded
    before(() => {
        if (existingUsers.length < 1) {
            throw new Error('Seed some users');
        }
    });
    describe('`getUserByEmail` (Login)', () => {
        it(`Wrong format email`, async () => {
            const user = await userRepo.getUserByEmail('some random string');
            (0, chai_1.expect)(user).not.to.exist;
        });
        it(`Non existing user`, async () => {
            const user = await userRepo.getUserByEmail('nonExisting@none.com');
            (0, chai_1.expect)(user).not.to.exist;
        });
        it('Non confirmed user, should return null', async function test() {
            const nonConfirmedUser = existingUsers.find((user) => !user.hash);
            if (!nonConfirmedUser) {
                return this.skip();
            }
            const user = await userRepo.getUserByEmail(nonConfirmedUser.email);
            return (0, chai_1.expect)(user).not.to.exist;
        });
        /**
         * EMail search should be case-insensitive
         */
        it(`Wrong email case, user with hash should be returned`, async () => {
            const seedUser = existingUsers[0];
            const user = await userRepo.getUserByEmail(seedUser.email.toUpperCase());
            (0, chai_1.expect)(user).to.exist;
            (0, chai_1.expect)(user?.hash).to.exist;
            (0, chai_1.expect)(user?.id).to.eq(seedUser.id);
        });
        it(`User login, user with hash should be returned`, async () => {
            const seedUser = existingUsers[0];
            const user = await userRepo.getUserByEmail(seedUser.email);
            (0, chai_1.expect)(user).to.exist;
            (0, chai_1.expect)(user?.hash).to.exist;
            (0, chai_1.expect)(user?.id).to.eq(seedUser.id);
        });
    });
    describe('getUserById` (Api Auth, Password Reset Request, Invitation - Optionally)', () => {
        // when `getNewUserById` **is not** implemented, negative tests are called as part of invitation tests
        if (userRepo.getNewUserById) {
            declareNegativeTestsForGetUserByIdMethod(userRepo.getUserById.bind(userRepo), existingUsers);
        }
        else {
            // only test non-confirmed user when `getNewUserById` is not implemented,
            // since only then should `getUserById` return non-confirmed user
            it('Non confirmed user', async function test() {
                const nonConfirmedUser = existingUsers.find((user) => !user.hash);
                if (!nonConfirmedUser) {
                    return this.skip();
                }
                const user = await userRepo.getUserById(nonConfirmedUser.id);
                return (0, chai_1.expect)(user).not.to.exist;
            });
        }
        it(`getUserById, user with hash should be returned`, async () => {
            const seedUser = existingUsers[0];
            const user = await userRepo.getUserById(seedUser.id);
            (0, chai_1.expect)(user).to.exist;
            (0, chai_1.expect)(user?.hash).to.exist;
            (0, chai_1.expect)(user?.id).to.eq(seedUser.id);
        });
    });
    describe('`getNewUserById` (Invitation))', () => {
        if (userRepo.getNewUserById) {
            // `getNewUserById` method should not return already confirmed user
            it('Get already confirmed user by ID, should return null', async () => {
                const seedUser = existingUsers[0];
                const user = await userRepo.getNewUserById?.(seedUser.id);
                (0, chai_1.expect)(user).not.to.exist;
            });
            declareInvitationTests(userRepo.getNewUserById.bind(userRepo), existingUsers);
        }
        else {
            declareInvitationTests(userRepo.getUserById.bind(userRepo), existingUsers);
        }
    });
    describe('`updatePassword` (Password Reset)', () => {
        const newPass = 'newPassword123.';
        it('Non existing user, should throw error', () => {
            let promise;
            if (typeof existingUsers[0].id === 'number') {
                promise = userRepo.updateUserPassword(99999, newPass);
            }
            else {
                promise = userRepo.updateUserPassword((0, uuid_1.v4)(), newPass);
            }
            promise
                .then(() => {
                throw new Error('was not supposed to succeed');
            })
                .catch((err) => {
                (0, chai_1.expect)(err).to.exist;
            });
        });
        it(`Wrong format id`, () => {
            userRepo
                .updateUserPassword('some random string', newPass)
                .then(() => {
                throw new Error('was not supposed to succeed');
            })
                .catch((err) => {
                (0, chai_1.expect)(err).to.exist;
            });
        });
        it('Non confirmed user', function test() {
            const nonConfirmedUser = existingUsers.find((user) => !user.hash);
            if (!nonConfirmedUser) {
                return this.skip();
            }
            return userRepo
                .updateUserPassword(nonConfirmedUser.id, newPass)
                .then(() => {
                throw new Error('was not supposed to succeed');
            })
                .catch((err) => {
                (0, chai_1.expect)(err).to.exist;
            });
        });
        it(`Update user password`, async () => {
            const user = existingUsers[0];
            const oldUser = await userRepo.getUserByEmail(user.email);
            const oldHash = oldUser?.hash;
            // fake new pass hash using uuid
            const randomPasswordHash = (0, uuid_1.v4)();
            await userRepo.updateUserPassword(user.id, randomPasswordHash);
            const updatedUser = await userRepo.getUserByEmail(user.email);
            (0, chai_1.expect)(updatedUser?.hash).not.to.eq(oldHash);
            // updateUserPassword needs password hash
            (0, chai_1.expect)(updatedUser?.hash).to.eq(randomPasswordHash);
        });
    });
}
exports.declareUserRepositoryTests = declareUserRepositoryTests;
