import { ID, IUserRepository } from '@goodrequest/passport-jwt-wrapper'
import { col, fn, Op, where, WhereOptions } from 'sequelize'
import { createSlug } from './utils/helpers'
import { BaseUserModel } from './models/baseUser'

// eslint-disable-next-line import/prefer-default-export
export class UserRepository<IDType extends ID, M extends typeof BaseUserModel<IDType>> implements IUserRepository<IDType> {
	user: M

	constructor(user: M) {
		this.user = user
	}

	async getUsers(limit: number, page: number, sort: string, search?: string): Promise<BaseUserModel<IDType>[]> {
		const userConditions: { [Op.and]: WhereOptions[] } = {
			[Op.and]: []
		}

		if (search) {
			const slugValue = createSlug(search)
			const searchConditions = slugValue.split('-').map((slug: string) =>
				where(fn('CONCAT_WS', '-', col('user.emailSlug'), col('user.nameSlug'), col('user.surnameSlug'), col('user.phoneSlug')), {
					[Op.iLike]: `%${slug}%`
				})
			)
			userConditions[Op.and].push(...searchConditions)
		}

		const offset = page * limit - limit

		const users = await this.user.findAll({
			attributes: ['id', 'name', 'surname', 'email', 'permission', 'confirmedAt'],
			where: userConditions,
			offset,
			limit,
			order: [['id', 'ASC']]
		})

		return users
	}

	async getUserByEmail(email: string): Promise<BaseUserModel<IDType> | undefined> {
		const user = await this.user.findOne({
			attributes: ['id', 'name', 'surname', 'email', 'permission', 'confirmedAt', 'hash'],
			where: {
				email: {
					[Op.iLike]: email
				},
				confirmedAt: {
					[Op.not]: null
				},
				hash: {
					[Op.not]: null
				}
			}
		})

		return user ?? undefined
	}

	async getUserById(id: IDType): Promise<BaseUserModel<IDType> | undefined> {
		const user = await this.user.findByPk(id, {
			attributes: ['id', 'name', 'surname', 'email', 'permission', 'confirmedAt', 'hash']
		})

		return user ?? undefined
	}

	async updateUserPassword(id: IDType, newPasswordHash: string): Promise<BaseUserModel<IDType>> {
		const user = await this.user.findByPk(id, {
			attributes: ['id', 'hash']
		})

		if (!user) {
			throw new Error('error:User was not found')
		}

		const transaction = await this.user.sequelize?.transaction()

		try {
			const updatedUsers = await user.update(
				{
					hash: newPasswordHash,
					updatedBy: id
				},
				{
					transaction
				}
			)

			if (transaction) {
				await transaction.commit()
			}

			return updatedUsers
		} catch (e) {
			if (transaction) {
				await transaction.rollback()
			}

			throw e
		}
	}

	/**
	 * return user only if he has no password set -- is not registered
	 * @param userID
	 */
	async getNewUserById(userID: IDType): Promise<BaseUserModel<IDType> | undefined> {
		const user = await this.user.findOne({
			where: {
				id: userID,
				hash: { [Op.is]: null }
			}
		})

		return user ?? undefined
	}
}
