import { expect } from 'chai'

import { createHash } from '@goodrequest/passport-jwt-wrapper'

import { PERMISSION } from '../../src/utils/enums'
import { UserRepository } from '../../src'

import { models } from '../db/models'
import { declareUserRepositoryTests } from '../declareTests'
import { IntUserModel } from '../db/models/int-user'
import { password, userData } from '../utils/helper'

const { UserInt } = models

/* eslint-disable @typescript-eslint/no-unused-expressions */
describe(`Big Int User`, () => {
	const userRepo = new UserRepository(UserInt)
	describe('Good seeds', () => {
		const dbUsers: IntUserModel[] = []

		before(async () => {
			const hash = await createHash(password)

			const mappedData = userData.map((data) => ({
				email: data.email,
				confirmedAt: data.isConfirmed ? Date.now() : null,
				hash: data.isConfirmed ? hash : null,
				permission: data.permission
			}))

			const seedUsers = await UserInt.bulkCreate(mappedData)
			dbUsers.push(...seedUsers)
		})

		declareUserRepositoryTests(userRepo, dbUsers)
	})

	describe('Wrong seeds', () => {
		beforeEach(async () => {
			await UserInt.truncate()
		})

		it('no seeds', async () => {
			const user = await userRepo.getUserById(1)
			expect(user).not.to.exist
		})

		it('`getUserById` More users with same email', async () => {
			const hash = await createHash(password)

			const data = {
				email: 'user@goodrequest.com',
				permission: PERMISSION.USER,
				confirmedAt: Date.now(),
				hash
			}

			const [seedUser1, seedUser2] = await UserInt.bulkCreate([
				data,
				{
					...data,
					email: 'user@goodrequest.com'
				}
			])

			const user = await userRepo.getUserById(seedUser1.id)
			const user2 = await userRepo.getUserById(seedUser2.id)
			expect(user).to.exist
			expect(user2).to.exist
			expect(user?.id).not.to.eq(user2?.id)
			expect(user?.email).to.eq(user2?.email)
		})

		it('`getNewUserById` More users with same email', async function test() {
			if (!userRepo.getNewUserById) {
				return this.skip()
			}

			const data = {
				email: 'user@goodrequest.com',
				permission: PERMISSION.USER
			}

			const [seedUser1, seedUser2] = await UserInt.bulkCreate([
				data,
				{
					...data,
					email: 'user@goodrequest.com'
				}
			])

			const user = await userRepo.getNewUserById(seedUser1.id)
			const user2 = await userRepo.getNewUserById(seedUser2.id)
			expect(user).to.exist
			expect(user2).to.exist
			expect(user?.id).not.to.eq(user2?.id)
			return expect(user?.email).to.eq(user2?.email)
		})

		it('`getUserByEmail` More users with same email', async () => {
			const hash = await createHash(password)

			const data = {
				email: 'user@goodrequest.com',
				permission: PERMISSION.USER,
				confirmedAt: Date.now(),
				hash
			}

			const [seedUser1, seedUser2] = await UserInt.bulkCreate([
				data,
				{
					...data,
					email: 'user@goodrequest.com'
				}
			])

			const user = await userRepo.getUserByEmail(seedUser1.email)
			const user2 = await userRepo.getUserByEmail(seedUser2.email)
			expect(user).to.exist
			expect(user2).to.exist
			expect(user?.id).to.eq(user2?.id)
			expect(user?.email).to.eq(user2?.email)
		})
	})
})
