import { expect } from 'chai'
import { ID, IUserRepository } from '@goodrequest/passport-jwt-wrapper'
// eslint-disable-next-line import/no-extraneous-dependencies
import { v4 as uuidv4 } from 'uuid'
import { GetUserByIdFunction } from '@goodrequest/passport-jwt-wrapper/dist/src/types/interfaces'

export interface ITestUser<IDType extends ID> {
	id: IDType
	email: string
	hash?: string
}

/* eslint-disable @typescript-eslint/no-unused-expressions */
function declareNegativeTestsForGetUserByIdMethod<IDType extends ID>(getUserByID: GetUserByIdFunction<IDType>, existingUsers: ITestUser<IDType>[]) {
	it(`[${getUserByID.name}] Wrong format id`, async () => {
		getUserByID('some random string' as IDType)
			.then(() => {
				throw new Error('was not supposed to succeed')
			})
			.catch((err) => {
				expect(err).to.exist
			})
	})

	it(`[${getUserByID.name}] Non existing user`, async () => {
		let user
		if (typeof existingUsers[0].id === 'number') {
			user = await getUserByID(99999 as IDType)
		} else {
			user = await getUserByID(uuidv4() as IDType)
		}

		expect(user).not.to.exist
	})
}

function declareInvitationTests<IDType extends ID>(getUserByID: GetUserByIdFunction<IDType>, existingUsers: ITestUser<IDType>[]) {
	declareNegativeTestsForGetUserByIdMethod(getUserByID, existingUsers)

	it(`[${getUserByID.name}] Get non confirmed user by ID`, async function test() {
		const nonConfirmedUser = existingUsers.find((user) => !user.hash)
		if (!nonConfirmedUser) {
			return this.skip()
		}

		const user = await getUserByID(nonConfirmedUser.id)
		expect(user).to.exist
		return expect(user?.id).to.eq(nonConfirmedUser.id)
	})
}

export function declareUserRepositoryTests<IDType extends ID>(userRepo: IUserRepository<IDType>, existingUsers: ITestUser<IDType>[]) {
	// test if there are some users seeded
	before(() => {
		if (existingUsers.length < 1) {
			throw new Error('Seed some users')
		}
	})

	describe('`getUserByEmail` (Login)', () => {
		it(`Wrong format email`, async () => {
			const user = await userRepo.getUserByEmail('some random string')

			expect(user).not.to.exist
		})

		it(`Non existing user`, async () => {
			const user = await userRepo.getUserByEmail('nonExisting@none.com')

			expect(user).not.to.exist
		})

		it('Non confirmed user, should return null', async function test() {
			const nonConfirmedUser = existingUsers.find((user) => !user.hash)
			if (!nonConfirmedUser) {
				return this.skip()
			}

			const user = await userRepo.getUserByEmail(nonConfirmedUser.email)
			return expect(user).not.to.exist
		})

		/**
		 * EMail search should be case-insensitive
		 */
		it(`Wrong email case, user with hash should be returned`, async () => {
			const seedUser = existingUsers[0]
			const user = await userRepo.getUserByEmail(seedUser.email.toUpperCase())

			expect(user).to.exist
			expect(user?.hash).to.exist
			expect(user?.id).to.eq(seedUser.id)
		})

		it(`User login, user with hash should be returned`, async () => {
			const seedUser = existingUsers[0]
			const user = await userRepo.getUserByEmail(seedUser.email)

			expect(user).to.exist
			expect(user?.hash).to.exist
			expect(user?.id).to.eq(seedUser.id)
		})
	})

	describe('getUserById` (Api Auth, Password Reset Request, Invitation - Optionally)', () => {
		// when `getNewUserById` **is not** implemented, negative tests are called as part of invitation tests
		if (userRepo.getNewUserById) {
			declareNegativeTestsForGetUserByIdMethod<IDType>(userRepo.getUserById.bind(userRepo), existingUsers)
		} else {
			// only test non-confirmed user when `getNewUserById` is not implemented,
			// since only then should `getUserById` return non-confirmed user
			it('Non confirmed user', async function test() {
				const nonConfirmedUser = existingUsers.find((user) => !user.hash)
				if (!nonConfirmedUser) {
					return this.skip()
				}

				const user = await userRepo.getUserById(nonConfirmedUser.id)

				return expect(user).not.to.exist
			})
		}

		it(`getUserById, user with hash should be returned`, async () => {
			const seedUser = existingUsers[0]
			const user = await userRepo.getUserById(seedUser.id)

			expect(user).to.exist
			expect(user?.hash).to.exist
			expect(user?.id).to.eq(seedUser.id)
		})
	})

	describe('`getNewUserById` (Invitation))', () => {
		if (userRepo.getNewUserById) {
			// `getNewUserById` method should not return already confirmed user
			it('Get already confirmed user by ID, should return null', async () => {
				const seedUser = existingUsers[0]
				const user = await userRepo.getNewUserById?.(seedUser.id)
				expect(user).not.to.exist
			})

			declareInvitationTests<IDType>(userRepo.getNewUserById.bind(userRepo), existingUsers)
		} else {
			declareInvitationTests<IDType>(userRepo.getUserById.bind(userRepo), existingUsers)
		}
	})

	describe('`updatePassword` (Password Reset)', () => {
		const newPass = 'newPassword123.'
		it('Non existing user, should throw error', () => {
			let promise
			if (typeof existingUsers[0].id === 'number') {
				promise = userRepo.updateUserPassword(99999 as IDType, newPass)
			} else {
				promise = userRepo.updateUserPassword(uuidv4() as IDType, newPass)
			}

			promise
				.then(() => {
					throw new Error('was not supposed to succeed')
				})
				.catch((err) => {
					expect(err).to.exist
				})
		})

		it(`Wrong format id`, () => {
			userRepo
				.updateUserPassword('some random string' as IDType, newPass)
				.then(() => {
					throw new Error('was not supposed to succeed')
				})
				.catch((err) => {
					expect(err).to.exist
				})
		})

		it('Non confirmed user', function test() {
			const nonConfirmedUser = existingUsers.find((user) => !user.hash)
			if (!nonConfirmedUser) {
				return this.skip()
			}

			return userRepo
				.updateUserPassword(nonConfirmedUser.id, newPass)
				.then(() => {
					throw new Error('was not supposed to succeed')
				})
				.catch((err) => {
					expect(err).to.exist
				})
		})

		it(`Update user password`, async () => {
			const user = existingUsers[0]
			const oldUser = await userRepo.getUserByEmail(user.email)
			const oldHash = oldUser?.hash

			// fake new pass hash using uuid
			const randomPasswordHash = uuidv4()
			await userRepo.updateUserPassword(user.id, randomPasswordHash)

			const updatedUser = await userRepo.getUserByEmail(user.email)

			expect(updatedUser?.hash).not.to.eq(oldHash)
			// updateUserPassword needs password hash
			expect(updatedUser?.hash).to.eq(randomPasswordHash)
		})
	})
}
